@@ -31,6 +31,16 @@
 #include "vanessa_logger.h"
 #include "l7vs.h"
 
+/* Added headers for sending messages to l7vsagent */
+#include <stdio.h>
+#include <errno.h>
+#include <sys/msg.h>
+#include <string.h>
+#include <signal.h>
+
+/* Added header for getnameinfo() */
+#include <netdb.h>
+
 static int l7vs_config_callback(struct l7vs_iomux *iom, int flags);
 static void l7vs_config_dispatch(union l7vs_config_req *req,
                                  struct sockaddr_un *addr);
@@ -56,6 +66,12 @@ static int l7vs_config_send_response(struct sockaddr_un *addr,
                                      struct iovec *iov,
                                      int iovlen);
 
+/* For message queue / Trap */
+int check_process_l7vsagent(char *);
+static char * get_servicename(struct sockaddr_in *, int);
+char trap_message[BUFSIZ]; /* Assuming IP address, Port and module name */
+char *srvname, *srvname_temp;
+
 const char l7vs_config_sockname[] = L7VS_CONFIG_SOCKNAME;
 
 static struct l7vs_iomux l7vs_config_iomux;
@@ -323,7 +339,7 @@ l7vs_config_list_rs(union l7vs_config_req *arg,
                 l7vs_config_send_response(addr, iov, 1);
                 return;
         }
-        
+
         rsp.num = ret;
         if (ret == 0) {
                 iovlen = 1;
@@ -363,11 +379,18 @@ l7vs_config_add_vs(union l7vs_config_req *arg,
                 goto out;
         }
 
+        /* Keep the message ready to be sent */
+        srvname = get_servicename(&sarg->addr, 1);
+        sprintf(trap_message, "Add Virtual Service:%s module %s scheduler %s", srvname, sarg->protomod, sarg->schedmod);
+
         srv = l7vs_service_create(sarg, &rsp.code);
         if (srv == NULL) {
                 VANESSA_LOGGER_ERR("ADD_VS: failed to create a service");
         } else {
 		VANESSA_LOGGER_INFO("ADD_VS: IFRM001: created a service ");
+
+                /* Send Trap message to l7vsagent process */
+                check_process_l7vsagent(trap_message);
 	}
 
 out:
@@ -399,6 +422,10 @@ l7vs_config_del_vs(union l7vs_config_req *arg,
                 goto out;
         }
 
+        /* Keep the message ready to be sent */
+        srvname = get_servicename(&sarg->addr, 1);
+        sprintf(trap_message, "Delete Virtual Service:%s module %s scheduler %s", srvname, sarg->protomod, sarg->schedmod);
+
 	while (g_list_length(srv->dest_list) > 0) {
 		dest = (g_list_first(srv->dest_list))->data;
 		srv->dest_list = g_list_remove(srv->dest_list, dest);
@@ -409,6 +436,9 @@ l7vs_config_del_vs(union l7vs_config_req *arg,
 
 	VANESSA_LOGGER_INFO("DEL_VS: IFRM003: released virtual service");
 
+        /* Send Trap message to l7vsagent process */
+        check_process_l7vsagent(trap_message);
+
 out:
         l7vs_config_send_response(addr, &iov, 1);
 }
@@ -451,6 +481,11 @@ l7vs_config_edit_vs(union l7vs_config_req *arg,
                 l7vs_sched_unbind(sched_old, srv);
                 l7vs_sched_put(sched_old);
                 l7vs_sched_bind(sched, srv);
+
+        /* Send Trap message to l7vsagent process */
+        srvname = get_servicename(&sarg->addr, 1);
+        sprintf(trap_message, "Edit Virtual Service:%s module %s scheduler %s", srvname, sarg->protomod, sarg->schedmod);
+        check_process_l7vsagent(trap_message);
         }
 
 out:
@@ -488,11 +523,19 @@ l7vs_config_add_rs(union l7vs_config_req *arg,
                 goto out;
         }
 
+        /* Keep the message ready to be sent */
+        srvname = get_servicename(&req->darg.addr, 1);
+        srvname_temp = get_servicename(&sarg->addr, 1);
+        sprintf(trap_message, "Add Real Server:%s for Virtual Service %s module %s", srvname, srvname_temp, sarg->protomod);
+
         ret = l7vs_service_add_dest(srv, &req->darg);
         if (ret < 0) {
                 rsp.code = L7VS_CONFIG_ERR_NOMEM;
         }
 
+        /* Send Trap message to l7vsagent process */
+        check_process_l7vsagent(trap_message);
+
 out:
         l7vs_config_send_response(addr, &iov, 1);
 }
@@ -530,12 +573,20 @@ l7vs_config_del_rs(union l7vs_config_req *arg,
                 goto out;
         }
 
+        /* Keep the message ready to be sent */
+        srvname = get_servicename(&req->darg.addr, 1);
+        srvname_temp = get_servicename(&sarg->addr, 1);
+        sprintf(trap_message, "Delete Real Server:%s for Virtual Service %s module %s", srvname, srvname_temp, sarg->protomod);
+
         ret = l7vs_service_remove_dest(srv, &req->darg);
         if (ret < 0) {
                 /* XXX actually impossible */
                 rsp.code = L7VS_CONFIG_ERR_NORS;
         }
 
+        /* Send Trap message to l7vsagent process */
+        check_process_l7vsagent(trap_message);
+
 out:
         l7vs_config_send_response(addr, &iov, 1);
 }
@@ -574,6 +625,14 @@ l7vs_config_edit_rs(union l7vs_config_req *arg,
 
         dest->weight = req->darg.weight;
 
+	/* Send signal SIGUSR1 to l7vsagent process if required */
+	/*
+	srvname = get_servicename(&req->darg.addr, 1);
+	srvname_temp = get_servicename(&sarg.addr, 1);
+	sprintf(trap_message, "Edit Real Server:%s for Virtual Service %s module %s", srvname, srvname_temp, sarg->protomod);
+	check_process_l7vsagent(trap_message);
+	*/
+
 out:
         l7vs_config_send_response(addr, &iov, 1);
 }
@@ -592,6 +651,10 @@ l7vs_config_flush_vs(union l7vs_config_req *arg, struct sockaddr_un *addr)
         iov.iov_len = sizeof(rsp);
 
         l7vs_service_flush_all();
+
+        /* Send signal SIGUSR1 to l7vsagent process */
+        check_process_l7vsagent("Clear L7VS Table");
+
         l7vs_config_send_response(addr, &iov, 1);
 }
 
@@ -617,3 +680,104 @@ l7vs_config_send_response(struct sockaddr_un *addr,
 
         return ret;
 }
+
+/*
+This method checks whether the process l7vsagent - L7VS SNMP subagent
+*  is running or not.
+*  If l7vsagent is running, then the function puts the appropriate TRAP message in the
+*  message queue, sends the signal SIGUSR1 to the l7vsagent process and returns 1.
+*  If l7vsagent is not running, then the function simply returns 0 and exits
+*/
+int check_process_l7vsagent(char *trap_message)
+{
+        FILE *fp;
+        char l7vsagent_pid_folder[30];
+        char l7vsagent_string[100];
+        char l7vsagent_pid[10];
+
+        /* message related */
+        char trap_string[BUFSIZ];
+        key_t key;
+        int msgid;
+
+        /* Open the PID file to see whether the snmp agent process is running */
+        fp = fopen("/var/run/l7vsagent.pid", "r");
+        if (fp != NULL)
+                fscanf(fp, "%s", l7vsagent_pid);
+        else
+                return 0;
+
+        fclose(fp);
+
+        sprintf(l7vsagent_pid_folder, "/proc/%s/cmdline", l7vsagent_pid);
+
+        /* Confirm whether the existing PID file is not stale */
+        fp = fopen(l7vsagent_pid_folder, "r");
+        if (fp != NULL)
+                        fscanf(fp, "%s", l7vsagent_string);
+        else
+        return 0;
+
+        fclose(fp);
+
+        /* Using strstr, confirm that the PID file is not stale or does not
+        belong to some other process */
+        if (strstr(l7vsagent_string, "l7vsagent") != NULL)
+        {
+                /* Create key for the message queue */
+                key = ftok("/var/run/l7vsagent.pid", 'b');
+                if ((int) key == -1)
+                                return 0;
+
+                /* Create or Get the message ID for the queue */
+                msgid = msgget(key, 0666 | IPC_CREAT);
+                if (msgid == -1)
+                        return 0;
+
+                /* Send the message to the queue */
+                strcpy(trap_string, trap_message);
+                if (msgsnd(msgid, (void *) &trap_string, sizeof(trap_string), 0) == -1)
+                        return 0;
+
+                /* Sending SIGUSR1 to l7vsagent */
+				kill((pid_t) atoi(l7vsagent_pid), SIGUSR1);
+                return 1;
+        }
+
+        /* In case of any errors or something, just return 0 */
+        return 0;
+}
+
+/*
+*  Method added for getting the address and port as character strings.
+*  This method was copied from l7vsadm.c
+*/
+static char * get_servicename(struct sockaddr_in *addr, int isnumeric)
+{
+        char *sname;
+        char hostname[40], portname[20];
+        int flags;
+        int ret;
+
+        flags = 0;
+        if (isnumeric) {
+                flags = NI_NUMERICHOST | NI_NUMERICSERV;
+        }
+
+        ret = getnameinfo((struct sockaddr *)addr, sizeof(*addr),
+                          hostname, sizeof(hostname),
+                          portname, sizeof(portname), flags);
+        if (ret != 0) {
+                VANESSA_LOGGER_ERR_UNSAFE("getnameinfo: %s",
+                                          gai_strerror(ret));
+                return NULL;
+        }
+
+        ret = asprintf(&sname, "%s:%s", hostname, portname);
+        if (ret < 0) {
+                VANESSA_LOGGER_ERR("Could not allocate memory");
+                return NULL;
+        }
+
+        return sname;
+}
